/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.io;

import org.testng.Assert;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

public class IdentifyFileTest
{

  @Test(groups = { "Functional" }, dataProvider = "identifyFiles")
  public void Identify(String data, String expectedFileType)
  {
    String protocol = AppletFormatAdapter.FILE;
    IdentifyFile ider = new IdentifyFile();
    String actualFiletype = ider.Identify(data, protocol);
    Assert.assertEquals(actualFiletype, expectedFileType,
            "File identification Failed!");
  }

  @DataProvider(name = "identifyFiles")
  public Object[][] IdentifyFileDP()
  {
    return new Object[][] {
        { "examples/example.json", "JSON" },
        { "examples/plantfdx.fa", "FASTA" },
        { "examples/dna_interleaved.phy", "PHYLIP" },
        { "examples/2GIS.pdb", "PDB" },
        { "examples/rf00031_folded.stk", "STH" },
        { "examples/testdata/test.rnaml", "RNAML" },
        { "examples/testdata/test.aln", "CLUSTAL" },
        { "examples/testdata/test.pfam", "PFAM" },
        { "examples/testdata/test.msf", "MSF" },
        { "examples/testdata/test.pir", "PIR" },
        { "examples/testdata/test.html", "HTML" },
        { "examples/testdata/test.pileup", "PileUp" },
        { "examples/testdata/test.blc", "BLC" },
        { "examples/testdata/simplegff3.gff", "GFF v2 or v3" },
        { "examples/testdata/test.jvp", "Jalview" },
        { "examples/testdata/test.cif", "mmCIF" },
        {
            "examples/testdata/cullpdb_pc25_res3.0_R0.3_d150729_chains9361.fasta.15316",
            "FASTA" },

    // { "examples/testdata/test.amsa", "AMSA" },
    // { "examples/test.jnet", "JnetFile" },
    };
  }

}
